from typing import Optional, Set, ClassVar
from pydantic import BaseModel, Field, EmailStr, ConfigDict
from datetime import date, datetime

class UserProfileUpdate(BaseModel):
    """
    User Profile Update model for partial updates of user information.
    All fields are optional to support partial updates.
    """
    model_config = ConfigDict(extra="forbid")

    # Metadata as class variables
    VERSION: ClassVar[float] = 2.01
    CLASS_ORIGIN_AUTHOR: ClassVar[str] = "Russlan Ramdowar;russlan@ftredge.com"
    CLASS_ORGIN_DATE: ClassVar[datetime] = datetime(2024, 3, 15, 20, 15)
    CLASS_REVISION_DATE: ClassVar[datetime] = datetime(2024, 3, 15, 20, 15)

    # System fields
    schema_version: Optional[float] = Field(None, description="Version of this Class == version of DB Schema")
    email: Optional[EmailStr] = Field(None, description="Propagated from Firebase Auth")
    organizations_uids: Optional[Set[str]] = Field(None, description="Organization memberships")
    
    # Timestamps and audit
    creat_date: Optional[datetime] = None
    creat_by_user: Optional[str] = None
    updt_date: Optional[datetime] = None
    updt_by_user: Optional[str] = None
    
    # System identification
    aliases: Optional[Set[str]] = None
    provider_id: Optional[str] = None

    # User-editable fields
    username: Optional[str] = Field(None, max_length=50, pattern=r"^[a-zA-Z0-9_-]+$")
    dob: Optional[date] = None
    first_name: Optional[str] = Field(None, max_length=100)
    last_name: Optional[str] = Field(None, max_length=100)
    mobile: Optional[str] = Field(None, pattern=r"^\+?[1-9]\d{1,14}$")

    def model_dump(self, **kwargs):
        kwargs.setdefault('exclude_none', True)
        return super().model_dump(**kwargs)



